=head1 NAME

DBIx::Class::Manual::Store

=head1 SYNOPSIS

This document details how the Data::Store works.

=head1 EXAMPLES

  Fill me in here

=head1 OVERVIEW

Data::Store is an abstraction layer that sits over a physical datastore and
provides a common interface across various physical datastores.

=head1 DETAILS

Data::Store abstracts a physical datastore in a Hash of Collection of
Objects (or Hash[Collection[Objects]]). Each Collection[Objects] conforms to a
given minimal specification for each Collection. This specification is defined
as a Collection of Types (or Collection[Types]).

From the client code perspective, there is one virtual store which provides an
abstraction around the concrete stores which map to physical datastores.

=head1 SUPPORTED DATASTORES

=head2 Relational databases

Relational databases map extremely well to this abstraction. The schema is the
Hash and each table within it is the Collection. The rows map to the Objects and
the table definition maps to the Collection[Types].

=head2 Object databases

Object databases map extremely well to this abstraction. The schema is the Hash
and each object maps to the Object. The Collection is the group of all objects
that belong to a given class. The class is the Collection[Types], each Type
being an attribute within the class.

=head2 Document databases

Document databases, such as CouchDB and Tokyo Cabinet, map relatively well to
this abstraction. The datastore itself is the Hash. Should each document have a
Class (or Table) value provided, then groups of docuemnts that have a given
value for Class act as a Collection[Objects]. Otherwise, the entire schema is
the Collection[Objects]. The Types are whatever types you have chosen to define.

=head2 KiokuDB

KiokuDB is being treated separately from OODBs and DocDBs. From the outside,
KiokuDB acts as an OODB and, if it provides an index on the classname, can be
treated as such. Otherwise, it acts as a DocDB and should be treated as such.

=head2 Perl data structures

Fill me in here.

=head1 API

The API for a Store breaks into two parts. The first set of methods are
transformative - they clone the existing store, modify the clone, and return it.
The second set are commands - they clone the existing store and return a command
that operates on that clone. In all cases, the existing store (the invocant) is
B<NOT> changed in any way.

=head2 TRANSFORMATIVE METHODS

=head3 $store->combine( $other_store, [ $args ] )

This takes two stores and provides a store that provides a search across both.
In other words, it acts as a JOIN in SQL. The optional C< $args > may be
specified to provide join conditions (such as ON in SQL).

$other_store is mandatory. $args is optional and store-dependent.

=head3 $store->project( $args )

This will determine the available values for reading off the stream should this
store have a Command::Query build off of it. The project values default to '*',
or all available values from the store(s) used as the basis. This corresponds to
the SELECT clause of a SQL statement.

$args is mandatory and store-dependent.

=head3 $store->fold( $args )

This will allow the use of aggregation functions by specifying which values are
to be pivoted on when aggregating. This corresponds to the GROUP BY clause of a
SQL statement.

$args is mandatory and store-dependent.

=head2 COMMAND METHODS

For the Command API, please see L<DBIx::Class::Store::Command/>.

=head3 $store->query_command()

This returns a Command::Query object.

=head3 $store->insert_command()

This returns a Command::Insert object.

=head3 $store->update_command()

This returns a Command::Update object.

=head3 $store->delete_command()

This returns a Command::Delete object.

=head1 TODO

=over 4

=item * Add EXAMPLES

=item * Fill in Perl data structures

=back

=head1 AUTHOR(S)

robkinyon: Rob Kinyon C<< <rkinyon@cpan.org> >>

=head1 LICENSE

You may distribute this code under the same terms as Perl itself.

=cut
